/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.area.plot;

import nl.cloudfarming.client.model.Shape;
import com.vividsolutions.jts.io.ParseException;
import nl.cloudfarming.client.area.plot.pref.AreaCulturePreference;
import com.vividsolutions.jts.geom.MultiPolygon;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import nl.cloudfarming.client.geoviewer.events.LayerEventKey;
import nl.cloudfarming.client.geoviewer.events.NewLayerEvent;
import nl.cloudfarming.client.model.PlotFile;
import nl.cloudfarming.client.util.MD5Sum;
import nl.cloudfarming.client.util.ShapeFileCopier;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.opengis.feature.simple.SimpleFeature;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra
 */
public class ShapeFileProcessor {

    private final File shapeFile;
    ShapeFileCopier fileCopier = new ShapeFileCopier();
    ArrayList<File> shapeList = new ArrayList<File>();
    private static Log log = LogFactory.getLog(ShapeFileProcessor.class);
    private final LayerEventProducer layerEventProducer = new LayerEventProducer();


    private static final String DEFAULT_SHAPE_NAME = NbBundle.getMessage(ShapeFileProcessor.class, "shape_file_processor.default_shape_name");

    public ShapeFileProcessor(File shapeFile) {
        this.shapeFile = shapeFile;
    }

    public void process() {
        
        ShapefileDataStore shpDataStore = null;
        if (shapeFile.isFile()) {
            try {
                String checkSum = MD5Sum.getHex(MD5Sum.createChecksum(new FileInputStream(shapeFile)));
                PlotFile plotFile = new PlotFile();
                plotFile.setChecksum(checkSum);
                plotFile.setFileCreated(new Date(shapeFile.lastModified()));
                plotFile.setFileName(shapeFile.getName());
                plotFile.setFileRead(new Date());

                ImportLayer layer = new ImportLayer(plotFile);

                shpDataStore = new ShapefileDataStore(shapeFile.toURI().toURL());
                String typeName = shpDataStore.getTypeNames()[0];
                FeatureSource featureSource = shpDataStore.getFeatureSource(typeName);
                FeatureCollection result = featureSource.getFeatures();
                FeatureIterator iterator = result.features();
                while (iterator.hasNext()) {
                    SimpleFeature feature = (SimpleFeature) iterator.next();
                    Shape shape = parseShape(feature);
                    layer.addObject(new ShapeLayerObject(shape));
                }
                iterator.close();

                NewLayerEvent event = new NewLayerEvent(layer);
                layerEventProducer.triggerEvent(event);
            } catch (NoSuchAlgorithmException ex) {
                Exceptions.printStackTrace(ex);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
    }

    /**
     * 
     * @param payload
     * @return
     */
    public Shape parseShape(SimpleFeature payload) {
        int attIndexPlotPoly = payload.getFeatureType().indexOf(AreaCulturePreference.SHAPE_ATT_POLYGON.getValue());
        int attIndexPlotName = payload.getFeatureType().indexOf(AreaCulturePreference.SHAPE_ATT_NAME.getValue());
        MultiPolygon polygon = null;
        if (attIndexPlotPoly >= 0) {
            polygon = (MultiPolygon) payload.getAttribute(attIndexPlotPoly);
        }
        String shapeName = DEFAULT_SHAPE_NAME;
        if (attIndexPlotName >= 0) {
            shapeName = (String) payload.getAttribute(attIndexPlotName);
        }
        try {
            Shape shape = new Shape(polygon.toString());
            shape.setName(shapeName);
            shape.setType(Shape.ShapeType.SKIP);
            return shape;
        } catch (ParseException ex) {
            Exceptions.printStackTrace(ex);
            throw new RuntimeException(ex);
        }
    }

    private class LayerEventProducer extends BaseGuiEventProducer {

        public LayerEventProducer() {
            super(LayerEventKey.class);
        }
    }
}
